import { EventEmitter } from 'events';

interface DatabaseOptions {
    path?: string;
    inMemory?: boolean;
    createIfMissing?: boolean;
    encrypt?: boolean;
    encryptionMethod?: 'aes-256-cbc' | 'aes-256-gcm' | 'chacha20-poly1305';
    encryptionKey?: string;
    encryptionKDF?: 'pbkdf2' | 'scrypt' | 'argon2';
    autoFlush?: boolean;
    flushInterval?: number;
    cacheDocuments?: boolean;
    maxCacheSize?: number;
    enableIndexing?: boolean;
    schemaValidation?: 'strict' | 'warn' | 'ignore';
    schemaPath?: string;
    debug?: boolean;
    logLevel?: 'error' | 'warn' | 'info' | 'debug';
}
interface SchemaField {
    type: 'string' | 'number' | 'boolean' | 'object' | 'array' | 'date' | 'buffer';
    required?: boolean;
    default?: any;
    unique?: boolean;
    index?: boolean;
    min?: number;
    max?: number;
    pattern?: RegExp;
    enum?: any[];
    ref?: string;
    validate?: (value: any) => boolean | string;
}
interface Schema {
    [fieldName: string]: SchemaField;
}
interface Document {
    _id: string;
    _createdAt: Date;
    _updatedAt: Date;
    _version?: number;
    [key: string]: any;
}
interface DocumentMetadata {
    id: string;
    createdAt: Date;
    updatedAt: Date;
    size: number;
    checksum?: string;
    version?: number;
}
type QueryOperator = '$eq' | '$ne' | '$gt' | '$gte' | '$lt' | '$lte' | '$in' | '$nin' | '$exists' | '$regex' | '$text' | '$and' | '$or' | '$nor' | '$not';
interface QueryCondition {
    [operator: string]: any;
}
interface QueryFilter {
    [field: string]: any | QueryCondition;
}
interface QueryOptions {
    limit?: number;
    skip?: number;
    sort?: {
        [field: string]: 1 | -1;
    } | Array<[string, 1 | -1]>;
    projection?: {
        [field: string]: 0 | 1;
    };
    explain?: boolean;
    hint?: string;
    timeout?: number;
}
interface CollectionOptions {
    schema?: Schema;
    encrypt?: boolean;
    compression?: boolean;
    indexes?: IndexDefinition[];
    maxDocuments?: number;
    autoIndex?: boolean;
    maxCacheSize?: number;
}
interface IndexDefinition {
    fields: {
        [field: string]: 1 | -1;
    };
    unique?: boolean;
    sparse?: boolean;
    name?: string;
}
interface InsertResult {
    id: string;
    success: boolean;
    document: Document;
    insertedCount: number;
}
interface InsertManyResult {
    insertedIds: string[];
    insertedCount: number;
    success: boolean;
}
interface UpdateResult {
    modifiedCount: number;
    success: boolean;
    upsertedId?: string;
    upsertedCount?: number;
}
interface DeleteResult {
    deletedCount: number;
    success: boolean;
}
interface FindResult<T = Document> {
    documents: T[];
    total: number;
    hasMore: boolean;
    page?: number;
    totalPages?: number;
}
interface QueryBuilder$1<T = Document> {
    where(field: string, operator: QueryOperator, value: any): QueryBuilder$1<T>;
    and(field: string, operator: QueryOperator, value: any): QueryBuilder$1<T>;
    or(field: string, operator: QueryOperator, value: any): QueryBuilder$1<T>;
    sort(field: string, direction: 1 | -1): QueryBuilder$1<T>;
    limit(count: number): QueryBuilder$1<T>;
    skip(count: number): QueryBuilder$1<T>;
    select(fields: string[]): QueryBuilder$1<T>;
    execute(): Promise<FindResult<T>>;
    findOne(): Promise<T | null>;
    count(): Promise<number>;
}
interface Transaction {
    commit(): Promise<void>;
    rollback(): Promise<void>;
    collection(name: string): any;
}
interface DatabaseStats {
    collections: number;
    totalDocuments: number;
    totalSize: number;
    indexes: number;
    uptime: number;
}
interface DatabaseEvents {
    'document:inserted': (collection: string, document: Document) => void;
    'document:updated': (collection: string, document: Document) => void;
    'document:deleted': (collection: string, documentId: string) => void;
    'collection:created': (name: string) => void;
    'collection:dropped': (name: string) => void;
    error: (error: Error) => void;
}

/**
 * Collection-aware file storage engine
 */
declare class FileStorage {
    private basePath;
    private ensuredDirs;
    private readonly MAX_CONCURRENT_FILES;
    /**
     * @param basePath - Root directory where all collections will be stored
     */
    constructor(basePath: string);
    /** Ensure a directory exists */
    ensureDirectory(path: string): Promise<void>;
    /**
     * @param collectionPath - Collection folder relative to basePath
     * @param document - Document with metadata to write
     */
    writeDocument(collectionPath: string, document: Document): Promise<void>;
    /**
     * @returns The document or null if the file doesn't exist
     */
    readDocument(collectionPath: string, documentId: string): Promise<Document | null>;
    /**
     * @returns An array of documents
     */
    readAllDocuments(collectionPath: string): Promise<Document[]>;
    /**
     * @returns true if the document was deleted, false if it didn't exist
     */
    deleteDocument(collectionPath: string, documentId: string): Promise<boolean>;
    /**
     * @returns true if the document exists, false if it doesn't
     */
    documentExists(collectionPath: string, documentId: string): Promise<boolean>;
    /** Lightweight metadata fetch */
    getDocumentMetadata(collectionPath: string, documentId: string): Promise<DocumentMetadata | null>;
}

/**
 * @typeParam T - Shape of the documents stored in this collection
 */
declare class Collection<T = Document> {
    private documentOps;
    private queryOps;
    /**
     * @param name - Collection name
     * @param storage - Storage engine implementation
     * @param options - Optional collection-specific configuration
     */
    constructor(name: string, storage: FileStorage, options?: CollectionOptions);
    /**
     * @param data - Document data to insert
     * @returns Result containing the inserted document's ID and success status
     */
    insert(data: Partial<T>): Promise<InsertResult>;
    /**
     * @param documents - Array of document data to insert
     * @returns Result containing all inserted document IDs and success status
     */
    insertMany(documents: Partial<T>[]): Promise<InsertManyResult>;
    /**
     * @param filter - Query criteria to match documents for updating
     * @param updateData - Partial document data to apply to matched documents
     * @returns Result containing count of modified documents and success status
     */
    update(filter: QueryFilter, updateData: Partial<T>): Promise<UpdateResult>;
    /**
     * @param filter - Query criteria to match documents for updating
     * @param updateData - Document data to apply or insert if no matches
     * @returns Result containing modified/inserted count and success status
     */
    upsert(filter: QueryFilter, updateData: Partial<T>): Promise<UpdateResult>;
    /**
     * @param filter - Query criteria to match documents for deletion
     * @returns Result containing count of deleted documents and success status
     */
    delete(filter: QueryFilter): Promise<DeleteResult>;
    /**
     * @param filter - Query criteria to match a single document for deletion
     * @returns Result containing count of deleted documents (0 or 1) and success status
     */
    deleteOne(filter: QueryFilter): Promise<DeleteResult>;
    /**
     * @param definition - Index definition specifying fields and options
     */
    createIndex(definition: IndexDefinition): Promise<void>;
    /**
     * @param filter - Query criteria (MongoDB-like)
     * @param options - Pagination, sorting, projection
     */
    find(filter?: QueryFilter, options?: QueryOptions): Promise<FindResult<T>>;
    /**
     * @param filter - Query criteria to match documents
     * @returns First matching document or null if no matches found
     */
    findOne(filter?: QueryFilter): Promise<T | null>;
    /**
     * @param id - The document ID to search for
     * @returns Document with matching ID or null if not found
     */
    findById(id: string): Promise<T | null>;
    /**
     * @returns QueryBuilder instance for chaining query operations
     */
    query(): QueryBuilder$1<T>;
    /**
     * @param filter - Query criteria to match documents
     * @returns Number of documents matching the filter
     */
    count(filter?: QueryFilter): Promise<number>;
    /**
     * @returns True if collection is empty, false otherwise
     */
    isEmpty(): Promise<boolean>;
    /**
     * Clear the in-memory LRU cache for this collection to free memory
     */
    clearCache(): void;
    /**
     * @returns Object containing collection metrics and performance data
     */
    stats(): Promise<{
        totalDocuments: number;
        totalSize: number;
        indexes: number;
        cacheSize: number;
    }>;
    /**
     * Initialize the collection by setting up document and query operations
     */
    initialize(): Promise<void>;
}

/**
 * Main database instance that manages collections and provides the public API
 */
declare class NuboDB extends EventEmitter {
    private static instance;
    private options;
    private storage;
    private encryptionManager?;
    private collections;
    private isOpen;
    private startTime;
    private logger;
    /**
     * @param options - Database configuration
     */
    private constructor();
    /**
     * @param options - Required for first initialization
     */
    static getInstance(options?: DatabaseOptions): Promise<NuboDB>;
    /**
     * @param options - Database configuration
     */
    static create(options: DatabaseOptions): Promise<NuboDB>;
    /**
     * Initialize the database and create storage directories
     */
    open(): Promise<void>;
    /**
     * Clean up resources and close all collections
     */
    close(): Promise<void>;
    /**
     * @param name - Collection name
     * @param options - Optional collection-specific settings
     * @returns Collection instance for CRUD operations
     */
    collection<T = any>(name: string, options?: CollectionOptions): Collection<T>;
    /**
     * @param name - Collection name
     * @param schema - Optional schema for validation
     * @param options - Collection configuration
     */
    createCollection<T = any>(name: string, schema?: Schema, options?: CollectionOptions): Promise<Collection<T>>;
    /**
     * @param name - Collection to drop
     * @returns Success status
     */
    dropCollection(name: string): Promise<boolean>;
    /**
     * @returns Array of collection names currently loaded in memory
     */
    listCollections(): Promise<string[]>;
    /**
     * @param name - Name of the collection to check
     * @returns True if collection exists in memory, false otherwise
     */
    hasCollection(name: string): boolean;
    /**
     * @returns Database statistics including document counts, size, and uptime
     */
    getStats(): Promise<DatabaseStats>;
    /**
     * @returns Copy of the database options used during initialization
     */
    getOptions(): DatabaseOptions;
    /**
     * @returns True if database is open and ready, false otherwise
     */
    isDatabaseOpen(): boolean;
    /**
     * @returns Absolute or relative path to database storage directory
     */
    getPath(): string;
    /**
     * Clear all collection caches to free memory
     */
    clearCaches(): void;
    /**
     * @param backupPath - Destination for backup files
     */
    backup(backupPath: string): Promise<void>;
    /**
     * Perform database compaction to optimize storage
     */
    compact(): Promise<void>;
    private setupLogger;
    private log;
    on<K extends keyof DatabaseEvents>(event: K, listener: DatabaseEvents[K]): this;
    once<K extends keyof DatabaseEvents>(event: K, listener: DatabaseEvents[K]): this;
    off<K extends keyof DatabaseEvents>(event: K, listener: DatabaseEvents[K]): this;
}

/**
 * @typeParam T - Document type being queried
 */
declare class QueryBuilder<T = Document> {
    private filter;
    private options;
    private collection;
    /**
     * @param collection - Collection instance to query
     */
    constructor(collection: any);
    /**
     * @param field - Document field to filter on
     * @param operator - Comparison operator ($eq, $gt, etc.)
     * @param value - Value to compare against
     */
    where(field: string, operator: QueryOperator, value: any): QueryBuilder<T>;
    /**
     * @param field - Document field to filter on
     * @param operator - Comparison operator
     * @param value - Value to compare against
     */
    and(field: string, operator: QueryOperator, value: any): QueryBuilder<T>;
    /**
     * @param field - Document field to filter on
     * @param operator - Comparison operator
     * @param value - Value to compare against
     */
    or(field: string, operator: QueryOperator, value: any): QueryBuilder<T>;
    /**
     * @param field - Field to sort by
     * @param direction - 1 for ascending, -1 for descending
     */
    sort(field: string, direction: 1 | -1): QueryBuilder<T>;
    /**
     * @param count - Maximum number of documents to return
     */
    limit(count: number): QueryBuilder<T>;
    /**
     * @param count - Number of documents to skip
     */
    skip(count: number): QueryBuilder<T>;
    /**
     * @param fields - Array of field names to include
     */
    select(fields: string[]): QueryBuilder<T>;
    /**
     * @returns Query results containing documents, total count, and pagination info
     */
    execute(): Promise<FindResult<T>>;
    /**
     * @returns First matching document or null if no matches found
     */
    findOne(): Promise<T | null>;
    /**
     * @returns Number of documents matching the current query filters
     */
    count(): Promise<number>;
    /**
     * @returns True if at least one document matches, false otherwise
     */
    exists(): Promise<boolean>;
    /**
     * @param updateData - Partial document data to apply to the matched document
     * @returns Updated document or null if no matches found
     */
    findOneAndUpdate(updateData: Partial<T>): Promise<T | null>;
    /**
     * @returns Deleted document or null if no matches found
     */
    findOneAndDelete(): Promise<T | null>;
}

/**
 * Handles encryption and decryption of data using AES-256 with PBKDF2 key derivation
 */
declare class EncryptionManager {
    private key;
    private algorithm;
    /**
     * @param encryptionKey - Password/key for encryption
     * @param algorithm - Encryption algorithm (default: aes-256-cbc)
     */
    constructor(encryptionKey: string, algorithm?: string);
    /**
     * @param password - Password to derive key from
     * @param salt - Optional salt (generates random if not provided)
     */
    private deriveKey;
    /**
     * @param data - String to encrypt
     * @returns Encrypted data in format "iv:encrypted"
     */
    encrypt(data: string): string;
    /**
     * @param encryptedData - Encrypted data in "iv:encrypted" format
     * @returns Decrypted string
     */
    decrypt(encryptedData: string): string;
    /**
     * @param obj - Object to encrypt
     * @returns Encrypted JSON string
     */
    encryptObject(obj: any): string;
    /**
     * @param encryptedData - Encrypted JSON string
     * @returns Decrypted object
     */
    decryptObject(encryptedData: string): any;
}

/**
 * Document with system metadata
 */
type DocumentWithMetadata = Document & {
    _id: string;
    _createdAt: Date;
    _updatedAt: Date;
    _version?: number;
};
/**
 * @typeParam T - Document type for this collection
 */
declare abstract class BaseCollection<T = Document> {
    protected name: string;
    protected storage: FileStorage;
    protected encryptionManager?: EncryptionManager;
    protected schema?: Schema;
    protected options: CollectionOptions;
    protected cache: Map<string, T>;
    protected indexes: Map<string, Map<any, string[]>>;
    protected isInitialized: boolean;
    protected documentCount: number;
    /**
     * @param name - Collection name
     * @param storage - Storage engine instance
     * @param options - Collection configuration
     */
    constructor(name: string, storage: FileStorage, options?: CollectionOptions);
    /**
     * Load documents from storage, build indexes, and populate cache
     */
    initialize(): Promise<void>;
    /** Ensure collection is initialized */
    protected ensureInitialized(): Promise<void>;
    /** Get all documents from storage with decryption */
    protected getAllDocuments(): Promise<T[]>;
    /**
     * @param documents - Documents to index
     */
    protected buildIndexes(documents: T[]): Promise<void>;
    /**
     * @param document - Document being modified
     * @param operation - Type of operation (insert/update/delete)
     */
    protected updateIndexes(document: T & DocumentWithMetadata, operation: 'insert' | 'update' | 'delete'): Promise<void>;
    /**
     * @param document - Document to extract key from
     * @param fields - Fields to include in index key
     */
    protected extractIndexKey(document: T & DocumentWithMetadata, fields: {
        [field: string]: 1 | -1;
    }): any;
    /** Clear the document cache to free memory */
    clearCache(): void;
    /**
     * @returns Object with document count, size, index count, and cache size
     */
    stats(): Promise<{
        totalDocuments: number;
        totalSize: number;
        indexes: number;
        cacheSize: number;
    }>;
}

/**
 * @typeParam T - Document type for this collection
 */
declare class DocumentOperations<T = Document> extends BaseCollection<T> {
    private queryOps;
    /**
     * @param name - Collection name
     * @param storage - Storage engine instance
     * @param options - Collection configuration
     */
    constructor(name: string, storage: any, options?: any);
    /**
     * @param data - Document data to insert
     * @returns Insert result with generated ID and document
     */
    insert(data: Partial<T>): Promise<InsertResult>;
    /**
     * @param documents - Array of documents to insert
     * @returns Bulk insert result with IDs and count
     */
    insertMany(documents: Partial<T>[]): Promise<InsertManyResult>;
    /**
     * @param filter - Query filter to match documents
     * @param updateData - Fields to update in matching documents
     * @returns Update result with modified count
     */
    update(filter: QueryFilter, updateData: Partial<T>): Promise<UpdateResult>;
    /**
     * @param filter - Query filter to match documents
     * @param updateData - Data to update or insert
     * @returns Update result with upsert information
     */
    upsert(filter: QueryFilter, updateData: Partial<T>): Promise<UpdateResult>;
    /**
     * @param filter - Query filter to match documents for deletion
     * @returns Delete result with deleted count
     */
    delete(filter: QueryFilter): Promise<DeleteResult>;
    /**
     * @param filter - Query filter to match document for deletion
     * @returns Delete result with deleted count (0 or 1)
     */
    deleteOne(filter: QueryFilter): Promise<DeleteResult>;
    /**
     * @param definition - Index definition with fields and options
     */
    createIndex(definition: IndexDefinition): Promise<void>;
}

/**
 * @typeParam T - Document type for this collection
 */
declare class QueryOperations<T = Document> extends BaseCollection<T> {
    /**
     * @param filter - MongoDB-like query filter
     * @param options - Query options (sort, limit, skip, projection)
     * @returns Query result with documents and metadata
     */
    find(filter?: QueryFilter, options?: QueryOptions): Promise<FindResult<T>>;
    /**
     * @param filter - Query filter to match documents
     * @returns First matching document or null
     */
    findOne(filter?: QueryFilter): Promise<T | null>;
    /**
     * @param id - Document ID to lookup
     * @returns Document or null if not found
     */
    findById(id: string): Promise<T | null>;
    /** @returns QueryBuilder instance for this collection */
    query(): QueryBuilder$1<T>;
    /**
     * @param filter - Query filter to match documents
     * @returns Number of matching documents
     */
    count(filter?: QueryFilter): Promise<number>;
    /** @returns True if collection is empty */
    isEmpty(): Promise<boolean>;
    /**
     * @param documents - Documents to filter
     * @param filter - Query filter to apply
     */
    private filterDocuments;
    /**
     * @param document - Document to check
     * @param filterEntries - Query filter entries
     */
    private matchesFilter;
    /**
     * @param document - Document to check
     * @param operator - Logical operator
     * @param conditions - Array of conditions to evaluate
     */
    private matchesLogicalOperator;
    /**
     * @param value - Document field value
     * @param operators - Comparison operators to apply
     */
    private matchesComparisonOperators;
    /**
     * @param documents - Documents to sort
     * @param sort - Sort specification (field: direction)
     */
    private sortDocuments;
    /**
     * @param documents - Documents to project
     * @param projection - Field projection specification
     */
    private projectDocuments;
    /**
     * @param documents - Documents to filter
     * @param filter - Query filter to apply
     */
    private useIndexes;
}

/**
 * Base error class for all database-related errors
 */
declare class DatabaseError extends Error {
    readonly code: string;
    readonly details?: any;
    /**
     * @param message - Error message
     * @param code - Error code for programmatic handling
     */
    constructor(message: string, code: string, details?: any);
}
/**
 * Error thrown when collection operations fail
 */
declare class CollectionError extends DatabaseError {
    constructor(message: string, details?: any);
}
/**
 * Error thrown when document operations fail
 */
declare class DocumentError extends DatabaseError {
    constructor(message: string, details?: any);
}
declare class SchemaError extends DatabaseError {
    constructor(message: string, details?: any);
}
declare class EncryptionError extends DatabaseError {
    constructor(message: string, details?: any);
}
declare class StorageError extends DatabaseError {
    constructor(message: string, details?: any);
}

/**
 * @returns A unique string identifier
 */
declare function generateId(): string;
/**
 * @returns Current date and time
 */
declare function generateTimestamp(): Date;
/**
 * @returns Object with ID and timestamps
 */
declare function createDocumentMetadata(id?: string): {
    id: string;
    createdAt: Date;
    updatedAt: Date;
};

/**
 * @param data - Document data to validate
 * @param schema - Schema definition to validate against
 */
declare function validateSchema(data: any, schema: Schema): void;
/**
 * @param data - Document data
 * @param schema - Schema with default values
 * @returns Data with defaults applied
 */
declare function applyDefaults(data: any, schema: Schema): any;
/**
 * @param data - Raw document data
 * @returns Sanitized document data
 */
declare function sanitizeDocument(data: any): Omit<Document, '_id' | '_createdAt' | '_updatedAt'>;

/**
 * @param options - Database configuration options
 * @returns Promise resolving to a database instance
 */
declare function createDatabase(options: DatabaseOptions): Promise<NuboDB>;
declare const getDatabase: typeof NuboDB.getInstance;

export { BaseCollection, Collection, CollectionError, type CollectionOptions, DatabaseError, type DatabaseEvents, type DatabaseStats, type DeleteResult, type Document, DocumentError, type DocumentMetadata, DocumentOperations, type DocumentWithMetadata, EncryptionError, EncryptionManager, FileStorage, type FindResult, type IndexDefinition, type InsertManyResult, type InsertResult, NuboDB, QueryBuilder, type QueryBuilder$1 as QueryBuilderType, type QueryCondition, type QueryFilter, QueryOperations, type QueryOperator, type QueryOptions, type Schema, SchemaError, type SchemaField, StorageError, type Transaction, type UpdateResult, applyDefaults, createDatabase, createDocumentMetadata, NuboDB as default, generateId, generateTimestamp, getDatabase, sanitizeDocument, validateSchema };
